
# q-expansion of newform 242.10.a.h, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 242
weight = 10
poly_data = [9963, -1503, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[-2016, 34, 2], 9], [[1971, 98, -2], 3]]

hecke_ring_character_values = None
aps_data = [[16, 0, 0], [-26, -1, 0], [-830, -4, -1], [-2587, -30, 1], [0, 0, 0], [24871, 301, 6], [-212180, -523, -213], [-186864, 305, -462], [41021, 7798, -259], [-1119363, -261, 1760], [1172017, -1322, 2181], [7650979, 70735, -1986], [3589317, 74196, 7942], [2557386, -29992, 23030], [-9281445, -14736, -265], [-318872, -444160, -2415], [16751190, -84768, 14130], [-102322938, 73916, -56080], [-37312476, -1386625, 74946], [24050400, 1473252, -16196], [-68134349, -697361, -209633], [-239038099, 2094468, 19561], [-71500630, 1466947, 194992], [67343222, 3778585, 86893], [-10634592, 5020277, -43729], [-647415140, 2608382, 419618], [-542345948, -7897580, 463860], [809773298, -3945923, -398752], [-160751330, -4561676, 927575], [-94884683, -6228454, 720442], [-3083639252, 7107370, 564572], [-1345170232, 7851160, -915208], [2514642529, -11403751, 604641], [-3563800624, -3386715, 782906], [-3829577312, 9655874, 2907257], [884813496, -8665282, 2168896], [8536440815, -8502621, -3421609], [4263110058, 4854323, 4548584], [-2031946488, 40367622, -4544052], [-2359610600, 19832654, -4697546], [19202275702, 6204140, -423662], [11238394650, 18593522, -9094097], [-24863577672, 58733502, 5688536], [-15842611932, 64854425, -3823133], [-6967958413, -6995201, 14588390], [3788754680, -43749210, -1519704], [3443561624, 133276560, -1975084], [37936752700, 131050780, -18554944], [10216387698, -50497044, 26454438], [3275440688, -96278922, 11554897], [13231991027, -132083696, 45683864], [50030372065, 123849716, -2956827], [-23092653867, 279237737, 6968013], [-7824646016, -40345657, -54658974], [20272521808, 229643759, 13295191], [-74363756049, 174078330, -50052221], [-8736762275, -518197591, 40113504], [-2337840736, -298109844, -19797884], [-13654426708, 92849928, -41115867], [-19701377273, -269914225, 94007571], [-4009131966, -470921416, 9194138], [58506379670, -285625772, -12120621], [39483255018, 845998199, 81636772], [-52968434008, 102608314, -10147800], [-120896505202, -25891689, 4346525], [41610723348, -365741754, 119655506], [98640486614, -1518857184, -174662154], [104555800924, -1197029993, 51004043], [199607492038, -636850552, -111361610], [-188695254056, 154863718, -259722075], [191883060471, -2096167272, -118454880], [39886580751, 1586347092, 49606415], [231911665391, -127979892, -182554077], [-181171567422, -2055545908, -105868156], [-316232149720, 751467372, -80224856], [19390244464, 345281402, -272506052], [120884192175, -3421863375, -30123574], [438053093544, -1515761440, 200465589], [5329527395, -486504974, 554557964], [527678061633, -931344976, -59802038], [498181176174, 3073971411, -199200792], [14837665277, -1228626477, -120016732], [535426859376, -427065174, 505253812], [527749881596, 262350885, -338495821], [-551006238045, 61615256, -140648029], [346200552122, 1227169456, 108258962], [-71459817410, -1516616329, -504968087], [-411449466300, 1759503413, -324221089], [70943516479, 1632992921, 197939422], [558639913456, 1960313416, 540444], [-160548880626, 4937246136, -706795714], [-400649248656, -6041977500, 481568912], [348793831889, -3177712968, 928933825], [87991081912, 44773187, 720605722], [116391864086, 9860821596, -78868862], [416819227483, -9161761624, -754186353], [-169365996362, -781185076, -978742320], [-859963362885, 11330107191, 561894807], [-2076758858800, -2597247312, 222512928], [-240969446253, 16388251511, 1059061443], [-610577687086, -6725384892, -925737458], [-1471553105772, -12671509722, 62308690], [-2174716175698, 2860707517, -1785977728], [1675340478659, 18985363219, -868273209], [-1252372337076, 3492599165, -2161408982], [1209265314014, -2236568211, -2024628983], [-2752741886382, -1983999357, -742856684], [-1611631194786, -7200237351, 106515385], [-1033875582615, 1389595974, -1959549903], [-1613318790705, -26744313508, 1620345300], [692522834115, 5148584912, 3293849923], [-1829666413508, -3720998684, 736311029], [-2811251712812, -19685156545, 1234793495], [1158569049026, -14703783831, -1258914404], [-1614824942643, 2779604546, -1384234227], [1489362306855, -19642897026, 26578296], [3180027243146, -18074032269, -94252320], [-704205444608, 26025436388, 2645096804], [-4721244011037, -26536386873, 890624643], [1942682170800, -3880268589, 3884199098], [-916859411879, 16813575583, -4236542504], [3911692970795, 30525614679, 334221759], [4768547554331, 881683651, -2144598114], [1675176133998, 32319218577, 2385409648], [-2010465703094, 16905538708, -4920820998], [-2006668827867, -22674605493, -742800384], [4193872898746, -2882131556, 2018398956], [5130931771563, 35423115504, 2344147619], [2129666477371, 3812277484, 3623883099], [2618001763223, -44612094945, 1855620022], [-1226279847844, 15019822263, -3904326786], [5020402294271, -23263799912, -3123184053], [-4002382053940, 18436795800, 2694323296], [-1411593309295, -66691120533, 1421532056], [-191387458419, 43900308864, -4257095170], [381383162826, -23818253485, -1518285115], [-5471978669430, 28528638636, 1538662652], [9001163781588, 51524320016, -3350983640], [-11721646073748, 27800943522, 106600322], [7217789254655, -51247702793, 3028691707], [-5247317097310, -30058239108, 633676778], [284503095752, -17325064322, -7687313282], [3156553251920, 15222050214, 989130944], [-10079374950380, 3744243173, -6594711602], [542354478111, 42330230957, 5894642526], [1983575310075, 18881047020, 15909591795], [2160961497646, -103316018540, -1863619901], [-5497458950222, 39641508176, -4820850376], [2788625643630, 1274029724, -10727871622], [-12195748145399, -19486065664, -531982319], [-10782663638882, -36337338542, -10885279185], [-8126540989258, 136528604233, 6161659321], [-8264151239436, 108744037436, -2563517912], [-4049374570503, -62376656928, 10529393673], [21730430044032, -48745399180, 1958943036], [-11898161895204, -33540197316, 1787296828], [-11542073038060, -38024646414, -8485770528], [-2954012458494, 17845167567, 19267322877], [2129800514028, -48792356605, 13695220751], [-10015975067280, -52275684900, -14576144427], [-2693025090768, -30224662395, -18148840846], [-21703920424008, -51488253075, -5531876925], [-25488899184927, 6502604804, -19816978763], [26978736588642, -28169093715, -12475598996], [-13074174274733, 32679115934, -30414240100], [4827580064364, -129542168928, 24372445168], [18537080032616, 94069940964, 19119725140], [-129896361165, 243114752579, -11633148842], [-12469688547465, -69539481334, -3007262960], [-11651581579760, 78458211698, -17627545026], [-11863277871386, 237032896163, 18946304224], [14957298519794, -101169381276, 2159849884], [35867170143915, 120395616798, 904975167], [-28434153920485, 26782592244, 21574235264], [34432860016087, -52828747172, -18162972385], [-25551134992948, 111131871619, 10868707007], [-39385883884114, 114092248476, 8033757394], [-18888816315789, 79949165691, -8211783018], [-34432906492904, 3138395188, 3698820716], [-17574581087617, 92366523861, -20965793650], [3726349792611, 85561614272, 14815203935], [-8523205471998, 240639145761, 3199605648], [-28036322947886, -266503606232, 8554706380], [23849113969890, 37263386637, 6192333857], [27301879178436, 282710303064, -20098350380], [-16005313890377, -223458957493, 16152737623], [-4090963423174, 76994442378, 32967618547], [3145200730432, -196959912620, -31160226904], [6681847269523, 141386221822, -10471799128], [36448779431819, 239644087552, -7369104073], [-49965035315386, -111048945756, -5211094340], [26500397546088, -447946476597, -44335750874], [956078780104, -547959193820, 17347118196], [-21233644153614, -63766868694, 36265910163], [1860846233392, 2829771161, -6361070706], [-67590649690523, 194770289030, 7278671022], [4129527973609, -244053518186, -38535441738], [-71516673965752, 257149574610, -3027943266], [61377666386925, -237545168094, 27843170226], [-9536933471436, -554838513780, 20119950932], [-23493483377824, -428166564794, 6602078670], [36026825868876, 280291428134, 11367728604], [-15066113519001, -371475496525, 4969913178], [4941220994998, 310740024361, -53801493231], [31262695671126, 402419970804, 29903646994], [1783107560152, 576601819826, -54015919858], [-34054765707268, 649668814866, -11636245452], [-63434024857948, -341181420641, -47953093718], [30436565324955, -561231941064, -71480597808], [78750023880878, 228983991339, -62842671240], [-136769870447687, -23383898204, 24652776086], [-16281196281966, -225395369544, 30188296119], [13221495996888, 903608292386, -30554436596], [97944368293470, -48361992333, 52833786388], [136677790749855, 335802256782, -26876028681], [79058776888847, 424623580902, -25805535108], [-118884479874789, -15514643998, 37639986175], [-62505738764461, 293227587487, -14699642617], [260994814985, -325283266118, -42786106827], [63192766570313, -455487507377, -51787138492], [119530397875119, 818500010495, -26240560142], [11714958905919, 1141742589668, -31056747765], [32526316037231, -704204433446, 105487425600], [92800854216705, -580264210960, 109753308501], [-93680668646708, -198539389288, -161207168296], [-143113459486832, 871793286514, 8469418497], [10890493878488, 838347489571, -72126999933], [11229569367549, 77133306900, 153816117453], [149458649582841, -36315857638, -10915247703], [172531479421920, 78432286341, -44494609374], [92778867531002, -487134929214, 152714174139], [-186714687968706, -545987700748, -54193057102], [67157526921164, -344095037178, 205211195896], [194878067588705, 595473676246, -53879690758], [-152098829135134, 554007191148, 81106011342], [-199554658243605, 223575369186, -86791583105], [-200765081597287, 269466473505, 43273107633], [-116883632713701, 155679479709, 5994752086], [40280989021344, -176429463372, -23346346500], [100700746854284, 981858131062, -36107307376], [87008049460098, -154524568989, -119951539500], [-234512992405888, 159017992488, 93327513912], [-73675367050644, -564669676786, 143245572436], [-170659662053689, -23350161453, 46412917366], [-104686345928906, -2213895990724, 87286391612], [15287290219237, -652530264760, -8180117743], [134713810361125, -1908423469316, -126329931663], [-81787715435488, 321590821336, -102350424076], [84551239877366, -315819305307, 155078481276], [287717273190664, 211601431376, -29029662644], [-150800038754133, 943610271647, -65077748981], [-163604150230923, 1669260956274, 83451878194], [-154294977830524, 1001022413164, 146768320804], [-275014646735735, 1097221677754, 83644253810], [49735699502095, -2535635138827, -8042032826], [-37735641339222, 2039939514144, -59502828330], [-108683138487040, 1590055069626, 188587448185], [-189475735482954, 1142013022700, 81928811866], [360495002632038, 69581196000, 50684800659], [-115301419365179, 478589863513, -175610188539], [-242617009627575, -695317218178, 193995005777], [-49401858400986, 572028606021, -81581236628], [-121894397913788, -978818462486, -131005574603], [-338555567730605, -35035492241, -52885789693], [312867724171431, -563834142174, 129035423784], [24794645827892, -363641060091, -193935089254], [-197369823641428, 2145521287366, 27129492890], [58542850988343, -1097939493933, 315198289371], [-405425777185634, 1572796693195, -1193075352], [-182316595081928, -2922035041702, 245859856358], [-211115245457077, -1969961653989, 101223344810], [-154904677231040, -1555895707604, 59436701720], [-525930837057855, 183015251114, -169614661014], [202485479887941, 1702311659630, 77109093949], [253931850494181, 1365214300277, -220610702619], [308790207167168, -1873419635112, -70797987328], [310847638998260, 108690630520, 3998031512], [-64629870802744, -1158169707930, 236545373997], [-31557752406390, 2553928022528, -156990153968], [54017336321668, -1432073139565, 360120772154], [-15714313433076, 1779584374254, 93220800376], [69728420149656, -2861654769046, -203521059948], [214019201476089, 2037708858828, 65118097521], [-17596409883571, 242948638263, -218588399220], [-158336818578608, -1493103579347, 189880294774], [-51286420741197, -913482427890, -71384155557], [-669797095742853, 226262743919, -190653963537], [213843521332167, -1474159603653, -220687257337], [-174131933685521, 1960201331422, 102852969319], [-66495935936926, -1091335529663, 465334178309], [465819584339413, 456512076287, 238382666848], [562667934446886, -770730990069, -125958574412], [59337340617480, 1090399780575, 172986716939], [-170978007967884, 683374427640, -435294308396], [-180463309577490, -2493385616044, 132759818280], [477406325371138, -2935733465416, 13292279284], [106423007276205, -2095888153102, -306042943315], [102893225711044, -4913614821442, -254462520539], [549884931137988, -2197352373864, -98171568644], [-288282849406564, -3336446830479, -95776437834], [44970643682738, -4771360571616, -131999470504], [576393831766208, -2059660073198, 26041975757], [86544556670139, 2017520903168, -732359923857], [-29386753522758, -2537416252236, -331732333278], [-692508459549228, 937441921295, -294950143710], [313093280741242, 1328641791280, 273839843640], [123039345750672, -3588321772980, 11902605528], [47160195850598, -44469139836, 343394539719], [656401679585055, 3802463073702, -130219013769], [255069159173708, 2338903901388, 549849328445], [249066236714883, 4201296040254, 313625791579], [484849164343935, 3881720813367, -395707619438], [-295282130118774, -1456250609169, -344795511499], [164800122714958, -4301741120855, -58082570632], [18234857236148, -2429870886050, -352032830020], [-214675795591491, -3410977182838, 425482734566], [-172818654922130, -4111839505795, 581580196244], [283512046540420, -1486119632476, 975337026976], [877800587613859, -38119368653, 352404794063], [-603122324108358, -1250882139903, -567928982715], [394472650645672, 1363471385152, 676555708852], [545850664553592, 7323242365685, -231313545005], [364060258800503, 7618842404299, -750207723762], [-91226287630812, -1567251850852, 423393642624], [906498670400153, 2388323038972, -145099550638], [-377120441464045, 3014941008951, -817266203241], [193222604199892, -6804802664420, -464913698732], [-192312417571222, 145801220520, -15092219270], [77068857912240, -1041571597884, -112914905240], [-348768994649956, 3946913582356, -354888042811], [-856822642085360, 4959053404834, -142990167347], [745685827241190, -840577612872, 1071503472888], [-1030717431425980, 3415335139740, 688842911452], [-844203590774060, -4747992253381, 116569805974], [-1170673959764042, -3570261464873, -276330168360], [-26527319429628, -3438054998523, 828326592702], [-64736195565554, -6402253103306, -600521799401], [-986799753706313, 5312840863391, 615756709147], [526016561832679, 10622798149228, -276255200676], [-1053954229931772, -2772137047744, -549474886232], [1170559433946063, 2773381380923, 470118280954], [1160374362377094, 4360723687044, -470439273836], [-237292225961758, 5396015823328, -1233197083473], [110824004364852, 5246411983148, -691775795264], [-520976204774301, 9041702561055, 719401016883], [-641172281995300, 3219995500552, 57785071716], [-95571640148589, -3554144915911, 904010249126], [-163459962600232, 603900412173, 479254574110], [-184897715753048, 3547752579488, -1350161843968], [1293497864725952, -5283531096260, -725905178759], [-584885618565460, -3375226856745, 221426364454], [-1573033475275015, -1872917246895, -720643207543], [433435829289801, -9309779438763, -503017081620], [-422231135966288, -6056212838960, -498224303432], [536037962832868, -2754920473580, 443584266429], [287873118887246, 116131019725, 841221261045], [12461811540883, 7646958823226, -138044148593], [387790838028472, 276928568540, -463186485748], [813141875629740, 2095748764389, 563930505603], [1174222432382336, 4879704837490, -188464090344], [348778377057650, 2989003559808, 710411277212], [-141741517964665, -6003836629085, 891305799355], [552361264327664, 12834485706190, -847573338076], [288985171774720, -1176319323967, -1483409766314], [419243699794502, -1498235142369, 437159544404], [-1012039469024032, 3904344268887, 25290996947], [-1303792788375530, 3743549787194, 1053195009759], [-1105052029119764, 10651691469238, -731591187316], [-789652437206953, -1953229724178, 363749980316], [-288323974694904, 2500743567501, 1595694663258], [-1573650136097456, -1347576585140, -727085852280], [360433059446169, 9168705170244, -553003582647], [399887385338898, 1742356207026, -794589743953], [379094082540244, -4025834344250, 369240695276], [1081266631711013, -1774977993687, -141280415208], [-653757228752552, 18962154240707, 67098353842], [445529788913495, -17320330561154, -493697943057], [-1608067662418549, 912485143476, -1438668220172], [-1893959506684738, -8012726370089, 1211442813665], [1472406909480725, -4894754619495, 612705804469], [-275914465507964, 14107291022342, 50129018770], [620479518539229, -1810826832672, -1688723070338], [173780816238956, 1586078405082, -2538408133280], [2772851292139390, 70265553428, -704041906780], [552947483227742, 2855570462577, 1671046175900], [-2806632620424009, 1388587691184, -897298793517], [255450461969733, -276697516186, -833082480195], [1250656571076252, -5941581081292, -1874665528275], [174348935325550, 14028132238792, -1669109228002], [558605150316588, 17809701633702, -1078218929792], [-922660530853785, -21072995281789, 970466272531], [-661511618048394, -15281188704804, -225528422664], [-223804801138990, -11467095646859, 551110505312], [-178645000219580, 7054887262567, 1522295748398], [-469979700137264, -12094677706102, 1907571513364], [523677118704863, 5486620901379, 2368857686179], [655333928419456, -17807943683114, -1925079275680], [1185917114071916, -7772940110435, 115459194535], [-704569285602458, 20691023210425, 1550324719268], [-1766618196547578, 850368427974, -488896938165], [4520870924255298, -3259591813678, -478075804045], [-931419079321593, 5488861881522, 1622002712544], [1471773629497560, -9656756710732, 433676632576], [-387383508230254, -13329594485264, -301555907272], [4492167682736639, 4741445434963, -257291724385], [-1632074129000851, -12140290805538, 1810314883729], [2336550405546716, 1436764248414, 2499880610129], [-1632174021645663, -9020200169400, 2099195093342], [1196768427749370, 12066269296235, -96596169844], [246837082229528, -6011166283376, 949642907016], [-1874433995955829, 1839389043252, -3729679537140], [-1463197617580213, 12471378066505, 478855374790], [-2346482767828878, -16445636704653, 150291113764], [2878412192461702, -17897571973241, 136477738804], [1221987904734378, 12075379873625, -555470375919], [588724937336872, 7733185363274, -1940889669099], [-2544784504583759, -15848481527146, 581123330713], [-3882808979350419, -1880296359106, -3046896485767], [4344653087314855, -14112918862333, -842860048861], [-3275330477190147, 2015493110718, -2305430614691], [1320806946059403, 61619516829, -1329564679430], [-861847602450668, -12638508916814, -884070377022], [713488647422269, -11459491293682, -644836103663], [-1513012927184820, -1610933786940, 1106744786532], [126320152195670, -22533571454925, 1036864809101], [1338471844310085, -11668356265065, 752266090280], [-1394262478944690, 29651593159848, -60766660270], [2024082432063319, -4921277275585, 1350040459795], [-3431176277853498, 8114416450100, -2835085555658], [2827869842752656, -29153732438502, -804670780120]]
